package java.util.logging;

import java.io.OutputStream;
import java.util.HashSet;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.nio.channels.OverlappingFileLockException;
import java.nio.file.FileAlreadyExistsException;
import java.nio.file.NoSuchFileException;
import java.nio.file.LinkOption;
import java.nio.file.StandardOpenOption;
import java.nio.file.OpenOption;
import java.nio.file.Paths;
import java.nio.file.Files;
import java.nio.file.Path;
import java.io.IOException;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.util.Set;
import java.io.File;
import java.nio.channels.FileChannel;

public class FileHandler extends StreamHandler
{
    private FileHandler.MeteredStream meter;
    private boolean append;
    private int limit;
    private int count;
    private String pattern;
    private String lockFileName;
    private FileChannel lockFileChannel;
    private File[] files;
    private static final int DEFAULT_MAX_LOCKS = 100;
    private static int maxLocks;
    private static final Set<String> locks;
    
    private void open(File file, boolean append) throws IOException {
        int n = 0;
        if (append) {
            n = (int)file.length();
        }
        this.setOutputStream(this.meter = new FileHandler.MeteredStream(new BufferedOutputStream(new FileOutputStream(file.toString(), append)), n));
    }
    
    private void configure() {
        LogManager logManager = LogManager.getLogManager();
        String name = this.getClass().getName();
        this.pattern = logManager.getStringProperty(name + ".pattern", "%h/java%u.log");
        this.limit = logManager.getIntProperty(name + ".limit", 0);
        if (this.limit < 0) {
            this.limit = 0;
        }
        this.count = logManager.getIntProperty(name + ".count", 1);
        if (this.count <= 0) {
            this.count = 1;
        }
        this.append = logManager.getBooleanProperty(name + ".append", false);
        this.setLevel(logManager.getLevelProperty(name + ".level", Level.ALL));
        this.setFilter(logManager.getFilterProperty(name + ".filter", null));
        this.setFormatter(logManager.getFormatterProperty(name + ".formatter", new XMLFormatter()));
        try {
            this.setEncoding(logManager.getStringProperty(name + ".encoding", null));
        }
        catch (Exception ex) {
            try {
                this.setEncoding(null);
            }
            catch (Exception ex2) {}
        }
    }
    
    public FileHandler() throws IOException, SecurityException {
        this.checkPermission();
        this.configure();
        this.openFiles();
    }
    
    public FileHandler(String pattern) throws IOException, SecurityException {
        if (pattern.length() < 1) {
            throw new IllegalArgumentException();
        }
        this.checkPermission();
        this.configure();
        this.pattern = pattern;
        this.limit = 0;
        this.count = 1;
        this.openFiles();
        return;
    }
    
    public FileHandler(String pattern, boolean append) throws IOException, SecurityException {
        if (pattern.length() < 1) {
            throw new IllegalArgumentException();
        }
        this.checkPermission();
        this.configure();
        this.pattern = pattern;
        this.limit = 0;
        this.count = 1;
        this.append = append;
        this.openFiles();
        return;
    }
    
    public FileHandler(String pattern, int limit, int count) throws IOException, SecurityException {
        if (limit < 0 || count < 1 || pattern.length() < 1) {
            throw new IllegalArgumentException();
        }
        this.checkPermission();
        this.configure();
        this.pattern = pattern;
        this.limit = limit;
        this.count = count;
        this.openFiles();
        return;
    }
    
    public FileHandler(String pattern, int limit, int count, boolean append) throws IOException, SecurityException {
        if (limit < 0 || count < 1 || pattern.length() < 1) {
            throw new IllegalArgumentException();
        }
        this.checkPermission();
        this.configure();
        this.pattern = pattern;
        this.limit = limit;
        this.count = count;
        this.append = append;
        this.openFiles();
        return;
    }
    
    private boolean isParentWritable(Path path) {
        Path path2 = path.getParent();
        if (path2 == null) {
            path2 = path.toAbsolutePath().getParent();
        }
        return path2 != null && Files.isWritable(path2);
    }
    
    private void openFiles() throws IOException {
        LogManager.getLogManager().checkPermission();
        if (this.count < 1) {
            throw new IllegalArgumentException("file count = " + this.count);
        }
        if (this.limit < 0) {
            this.limit = 0;
        }
        FileHandler.InitializationErrorManager errorManager = new FileHandler.InitializationErrorManager();
        this.setErrorManager(errorManager);
        int n = -1;
        while (++n <= FileHandler.maxLocks) {
            this.lockFileName = this.generate(this.pattern, 0, n).toString() + ".lck";
            Label_0423: {
                synchronized (FileHandler.locks) {
                    if (!FileHandler.locks.contains(this.lockFileName)) {
                        Path value = Paths.get(this.lockFileName, new String[0]);
                        FileChannel lockFileChannel = null;
                        int n2 = -1;
                        boolean n3 = false;
                        while (lockFileChannel == null && n2++ < 1) {
                            try {
                                lockFileChannel = FileChannel.open(value, new OpenOption[] { StandardOpenOption.CREATE_NEW, StandardOpenOption.WRITE });
                                n3 = true;
                            }
                            catch (FileAlreadyExistsException ex) {
                                if (Files.isRegularFile(value, new LinkOption[] { LinkOption.NOFOLLOW_LINKS }) && this.isParentWritable(value)) {
                                    try {
                                        lockFileChannel = FileChannel.open(value, new OpenOption[] { StandardOpenOption.WRITE, StandardOpenOption.APPEND });
                                    }
                                    catch (NoSuchFileException ex2) {}
                                    catch (IOException ex3) {
                                        break;
                                    }
                                }
                                else {
                                    break;
                                }
                            }
                        }
                        if (lockFileChannel != null) {
                            this.lockFileChannel = lockFileChannel;
                            int n4;
                            try {
                                n4 = (this.lockFileChannel.tryLock() != null);
                            }
                            catch (IOException ex4) {
                                n4 = n3;
                            }
                            catch (OverlappingFileLockException ex5) {
                                n4 = false;
                            }
                            if (!(n4 == 0)) {
                                FileHandler.locks.add(this.lockFileName);
                                break Label_0423;
                            }
                            this.lockFileChannel.close();
                        }
                    }
                }
                continue;
            }
            this.files = new File[this.count];
            for (int i = 0; i < this.count; ++i) {
                this.files[i] = this.generate(this.pattern, i, n);
            }
            if (this.append) {
                this.open(this.files[0], true);
            }
            else {
                this.rotate();
            }
            Exception lastException = errorManager.lastException;
            if (lastException != null) {
                if ((lastException instanceof IOException)) {
                    throw (IOException)lastException;
                }
                if ((lastException instanceof SecurityException)) {
                    throw (SecurityException)lastException;
                }
                throw new IOException("Exception: " + lastException);
            }
            else {
                this.setErrorManager(new ErrorManager());
                return;
            }
        }
        throw new IOException("Couldn't get lock for " + this.pattern + ", maxLocks: " + FileHandler.maxLocks);
    }
    
    private File generate(String s, int n, int n2) throws IOException {
        Object o = null;
        String s2 = "";
        int i = 0;
        boolean n3 = false;
        boolean n4 = false;
        while (i < s.length()) {
            int char1 = s.charAt(i);
            ++i;
            int lowerCase = '\0';
            if (i < s.length()) {
                lowerCase = Character.toLowerCase(s.charAt(i));
            }
            if (char1 == '/') {
                if (o == null) {
                    o = new File(s2);
                }
                else {
                    o = new File(o, s2);
                }
                s2 = "";
            }
            else {
                if (char1 == '%') {
                    if (lowerCase == 't') {
                        String pathname = System.getProperty("java.io.tmpdir");
                        if (pathname == null) {
                            pathname = System.getProperty("user.home");
                        }
                        o = new File(pathname);
                        ++i;
                        s2 = "";
                        continue;
                    }
                    else if (lowerCase == 'h') {
                        o = new File(System.getProperty("user.home"));
                        if (isSetUID()) {
                            throw new IOException("can't use %h in set UID program");
                        }
                        ++i;
                        s2 = "";
                        continue;
                    }
                    else {
                        if (lowerCase == 'g') {
                            s2 += n;
                            n3 = true;
                            ++i;
                            continue;
                        }
                        if (lowerCase == 'u') {
                            s2 += n2;
                            n4 = true;
                            ++i;
                            continue;
                        }
                        if (lowerCase == '%') {
                            s2 += "%";
                            ++i;
                            continue;
                        }
                    }
                }
                s2 += (char)char1;
            }
        }
        if (this.count > 1 && n3 == 0) {
            s2 = s2 + "." + n;
        }
        if (n2 > 0 && n4 == 0) {
            s2 = s2 + "." + n2;
        }
        if (s2.length() > 0) {
            if (o == null) {
                o = new File(s2);
            }
            else {
                o = new File(o, s2);
            }
        }
        return o;
    }
    
    private synchronized void rotate() {
        Level level = this.getLevel();
        this.setLevel(Level.OFF);
        super.close();
        for (int i = this.count - 2; i >= 0; --i) {
            File file = this.files[i];
            File dest = this.files[i + 1];
            if (file.exists()) {
                if (dest.exists()) {
                    dest.delete();
                }
                file.renameTo(dest);
            }
        }
        try {
            this.open(this.files[0], false);
        }
        catch (IOException ex) {
            this.reportError(null, ex, 4);
        }
        this.setLevel(level);
    }
    
    @Override
    public synchronized void publish(LogRecord logRecord) {
        if (!this.isLoggable(logRecord)) {
            return;
        }
        super.publish(logRecord);
        this.flush();
        if (this.limit > 0 && this.meter.written >= this.limit) {
            AccessController.doPrivileged(new PrivilegedAction<Object>() {
                @Override
                public Object run() {
                    FileHandler.access$100(FileHandler.this);
                    return null;
                }
            });
        }
        return;
    }
    
    @Override
    public synchronized void close() throws SecurityException {
        super.close();
        if (this.lockFileName == null) {
            return;
        }
        try {
            this.lockFileChannel.close();
        }
        catch (Exception ex) {}
        synchronized (FileHandler.locks) {
            FileHandler.locks.remove(this.lockFileName);
        }
        new File(this.lockFileName).delete();
        this.lockFileName = null;
        this.lockFileChannel = null;
        return;
    }
    
    private static native boolean isSetUID();
    
    static void access$100(FileHandler fileHandler) {
        fileHandler.rotate();
    }
    
    static {
        locks = new HashSet();
        FileHandler.maxLocks = (int)AccessController.doPrivileged(()/*  */ -> Integer.getInteger("jdk.internal.FileHandlerLogging.maxLocks", 100));
        if (FileHandler.maxLocks <= 0) {
            FileHandler.maxLocks = 100;
        }
    }
    
    private class MeteredStream extends OutputStream
    {
        final OutputStream out;
        int written;
        
        MeteredStream(OutputStream out, int written) {
            this.out = out;
            this.written = written;
        }
        
        @Override
        public void write(int n) throws IOException {
            this.out.write(n);
            ++this.written;
        }
        
        @Override
        public void write(byte[] b) throws IOException {
            this.out.write(b);
            this.written += b.length;
        }
        
        @Override
        public void write(byte[] b, int off, int len) throws IOException {
            this.out.write(b, off, len);
            this.written += len;
        }
        
        @Override
        public void flush() throws IOException {
            this.out.flush();
        }
        
        @Override
        public void close() throws IOException {
            this.out.close();
        }
    }
    
    private static class InitializationErrorManager extends ErrorManager
    {
        Exception lastException;
        
        @Override
        public void error(String s, Exception lastException, int n) {
            this.lastException = lastException;
        }
    }
}
